##################################################################################
#
#   Kaivo Public Software License
#
#   Copyright (c) 2001, Kaivo, Inc.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions are
#   met:
#
#       o Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#
#       o Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#
#       o Neither Kaivo nor the names of its contributors may be used to
#         endorse or promote products derived from this software without 
#         specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL KAIVO OR ITS CONTRIBUTORS BE LIABLE FOR 
#   ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
#   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
#   USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##################################################################################

"""DTML Document Site Search Extension"""

from string import join
from OFS.DTMLDocument import DTMLDocument
from StripTagParser import StripTags

#
# Define alternate methods to index DTML docs against
#

def text_content(self, title_weight=3):
    """Return the rendered raw-text output of the document if possible,
       stripping the html tags"""
    title = getattr(self.aq_base, 'title', '')
    title = (title + '\n') * title_weight
    try:
        return title + StripTags(self(self))
    except:
        return title + StripTags(self.read())
DTMLDocument.text_content = text_content

def summary(self, length=200):
    """Create a summary for the content"""
    return self.text_content(title_weight=0)[:length]
DTMLDocument.summary = summary

#
# Define event handlers for catalog awareness
#

def getCatalogId(self):
    """Return the name of the catalog used by this document"""
    return getattr(self, 'SITE_CATALOG', '')
DTMLDocument.getCatalogId = getCatalogId

def url(self):
    """Return the absolute object path"""
    return join(self.getPhysicalPath(),'/')
DTMLDocument.url = url

def index_object(self):
    """A common method to allow Findables to index themselves."""
    if hasattr(self, self.getCatalogId()) and \
       not getattr(self, 'NOT_CATALOGED', 0):
        getattr(self, self.getCatalogId()).catalog_object(self, self.url())
        self.is_indexed_ = 1
DTMLDocument.index_object = index_object

def unindex_object(self):
    """A common method to allow Findables to unindex themselves."""
    if hasattr(self, self.getCatalogId()):
        getattr(self, self.getCatalogId()).uncatalog_object(self.url())
        self.is_indexed_ = 0
DTMLDocument.unindex_object = unindex_object

def reindex_object(self):
    """Reindex the object in the Catalog"""
    if getattr(self, 'is_indexed_', 0): self.unindex_object()
    self.index_object()
DTMLDocument.reindex_object = reindex_object

def manage_afterAdd(self, item, container):
    self.index_object()
DTMLDocument.manage_afterAdd = manage_afterAdd

def manage_afterClone(self, item):
    self.index_object()
DTMLDocument.manage_afterClone = manage_afterClone

def manage_beforeDelete(self, item, container):
    self.unindex_object()
DTMLDocument.manage_beforeDelete = manage_beforeDelete

original_edit = DTMLDocument.manage_edit
def manage_edit(self,data,title,SUBMIT='Change',dtpref_cols='50',
                    dtpref_rows='20',REQUEST=None):
    """Edit object an reindex"""
    r = original_edit(self,data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST)
    self.reindex_object()
    return r
DTMLDocument.manage_edit = manage_edit

original_addProperty = DTMLDocument.manage_addProperty
def manage_addProperty(self, id, value, type, REQUEST=None):
    """Add property and reindex"""
    r = original_addProperty(self,id,value,type,REQUEST)
    self.reindex_object()
    return r
DTMLDocument.manage_addProperty = manage_addProperty

original_delProperties = DTMLDocument.manage_delProperties
def manage_delProperties(self, ids=None, REQUEST=None):
    """Delete properties and reindex"""
    r = original_delProperties(self, ids, REQUEST)
    self.reindex_object()
    return r
DTMLDocument.manage_delProperties = manage_delProperties

original_changeProperties = DTMLDocument.manage_changeProperties
def manage_changeProperties(self, REQUEST=None, **kw):
    """Update properties and reindex"""
    r = original_changeProperties(self, REQUEST, kw=kw)
    self.reindex_object()
    return r
DTMLDocument.manage_changeProperties = manage_changeProperties

original_editProperties = DTMLDocument.manage_editProperties
def manage_editProperties(self, REQUEST):
    """Edit Properties and reindex"""
    r = original_editProperties(self, REQUEST)
    self.reindex_object()
    return r
DTMLDocument.manage_editProperties = manage_editProperties
